package meerkat.voting;

import com.google.common.util.concurrent.FutureCallback;
import meerkat.protobuf.Voting.*;

/**
 * Created by hai on 23/02/16.
 */
public interface VotingBoothUI {

    enum VoterQuestionChoice {
        ANSWERED,
        SKIP,
        BACK,
        CANCEL
    }

    class VoterQuestionResponse {
        private final VoterQuestionChoice m_choice;
        private final String m_answer;

        public VoterQuestionResponse(VoterQuestionChoice choice, String answer) {
            assert ((choice == VoterQuestionChoice.ANSWERED && answer != null) ||
                    (choice != VoterQuestionChoice.ANSWERED && answer == null));
            m_choice = choice;
            m_answer = answer;
        }

        public VoterQuestionChoice getChoice () { return m_choice; }
        public String getAnswer () { return m_answer; }
    }


    enum FinalizeBallotChoice {
        CAST,
        AUDIT
    }

    // Voter scenario methods
    void newVoter (int requestId, FutureCallback<VotingBoothResult<Boolean>> callback);

    void chooseChannel (int requestId, BallotQuestion question, FutureCallback<VotingBoothResult<Integer>> callback);

    void askVoterQuestion (int requestId, int questionNumber, BallotQuestion question,
                           FutureCallback<VotingBoothResult<VoterQuestionResponse>> callback);

    void castOrAudit (int requestId, FutureCallback<VotingBoothResult<FinalizeBallotChoice>> callback);

    void showWaitForFinishScreen (int requestId, FutureCallback<VotingBoothResult<Boolean>> callback);


    // Admin scenario methods
    //TODO: the admin scenario still needs some more thinking



    // bad thing happened scenario
    void showErrorMessageAndHalt (int requestId, String errorMessage,
                                  FutureCallback<VotingBoothResult<Boolean>> callback);

    void showErrorMessageWithButtons (int requestId, String errorMessage, String[] buttonLabels,
                                      FutureCallback<VotingBoothResult<Integer>> callback);

}
